{% load i18n %}
<form method="post" enctype="multipart/form-data">
{% csrf_token %}
{{ wizard.form.media }}

{{ wizard.management_form }}
{% for form in wizard.forms%}
    {% if form.forms %}
       {{ form.management_form }}
        {% for form in form.forms %}
            {{ form.as_p }}
        {% endfor %}
    {% else %}
        {{ form.as_p }}
    {% endif %}
{% endfor %}

{% if wizard.steps.prev %}
<button name="wizard_goto_step" type="submit" value="{{ wizard.steps.first }}">{% trans "first step" %}</button>
<button name="wizard_goto_step" type="submit" value="{{ wizard.steps.prev }}">{% trans "prev step" %}</button>
{% endif %}
<input type="submit" name="submit" value="{% trans 'submit' %}" />
</form>
